from .DataReading.DataReader import ReadData
from .DataScaling.DataScaler import ScaleData
from .DataModeling.DataModeler import ModelData
import pandas as pd

import argparse
import warnings

warnings.filterwarnings('ignore')


p = argparse.ArgumentParser()
p.add_argument('--data_path', type=str, help='Mandatory. Path to the data file.')
p.add_argument('--target_var', type=str, help='Mandatory. Name or position of the target column.')
p.add_argument('--test_size', type=float, help='Optional. Size of the test set. Default is 0.2.')
p.add_argument('--cv', type=float, help='Optional. Number of cross validations to use when evaluating the model. Default is 5.')
p.add_argument('--path_to_save_metrics', type=str, help='Optional. Path to save the metrics. It must be a file with the extension .csv.'
                                                        'If not provided, the metrics will be printed in the console but not saved.')
args = p.parse_args()

if not args.data_path:
    raise ValueError('You must provide a path to the data file with the argument --data_path.')

if not args.target_var:
    raise ValueError('You must provide the name of the target column with the argument --target_var.')

data_reader = ReadData(args.data_path)
data_reader.read_data()


data_scaler = ScaleData(data_reader.data, target=args.target_var)
data_scaler.select_dtypes()
data_scaler.clean_data()
scaling_methods = data_scaler.create_scaling_methods_pool()


data_modeler = ModelData(data_scaler.X, data_scaler.y, test_size=args.test_size if args.test_size else 0.2)
models = data_modeler.create_models_pool()


results = {}

# Iterate over all the scaling methods and models
for scaling_method in scaling_methods:
    for model_name, model in models.items():
        print(f'Using {scaling_method} scaling method and {model_name} model.')

        # Scale the data
        xtr_scaled, xte_scaled = data_scaler.scale_data(scaler=scaling_methods[scaling_method], xtr=data_modeler.xtr, xte=data_modeler.xte)

        # Train the model and get the predictions
        predictions = data_modeler.train_algorithm_and_return_predictions(model, xtr_scaled=xtr_scaled, xte_scaled=xte_scaled)

        # Calculate the metrics
        metrics = data_modeler.evaluate_model(model=model, predictions=predictions, cv=args.cv if args.cv else 5)

        # Store the results
        results[(scaling_method, model_name)] = metrics


# Convert the results to a DataFrame
results_df = pd.DataFrame(results).T
print('\nThe obtained metrics have been the following:')
print(results_df)

if args.path_to_save_metrics:
    print(f'\nSaving metrics to {args.path_to_save_metrics}')
    results_df.to_csv(args.path_to_save_metrics)
    print('The metrics have been correctly saved.')






#__init__.py
from mlforall import DataReading, DataScaling, DataModeling

__all__ = [
    'DataReading',
    'DataScaling',
    'DataModeling'
]

__version__ = '0.3.1'

__doc__ = '''

## Description
**mlforall** is an open-source library aimed to developers that are beginners in the data analysis area but want to build powerful machine learning projects from the very beginning. The package offers a reliable, easy to use and well documented set of functions that drive the user through the most common steps of any machine learning projects, from data reading to model testing.

## Main features
These are some of the functionalities that mlforall offers:
1. File extension asbtraction when reading data (only supported for `.csv`, `.txt`, `.xlsx`, `.xlsx`, `.parquet` and `.npy`)
2. Automatic handling of non-numeric features and missing values.
3. A pool with almost all the data-scaling methods available and the most common ML models.
4. Automatic model evaluation and reporting.

'''




from distutils.core import setup
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()



setup(
  name = 'mlforall',         # How you named your package folder (MyLib)
  packages = ['mlforall'],   # Chose the same as "name"
  version = '0.3.1',      # Start with a small number and increase it with every change you make
  license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'Library that easily allows to create machine learning progress for more unexperiencied programmers.',   # Give a short description about your library
  author = 'Unai Torrecilla',                   # Type in your name
  author_email = 'unai.torrecilla@alumni.mondragon.edu',      # Type in your E-Mail
  url = 'https://github.com/UnaiTorrecilla/MLForAll',   # Provide either the link to your github or to your website
  download_url = 'https://github.com/UnaiTorrecilla/MLForAll/archive/refs/tags/v_031.tar.gz',    # I explain this later on
  long_description = long_description,
  long_description_content_type = 'text/markdown',
  keywords = ['Machine learning', 'Easy to use'],   # Keywords that define your package best
  install_requires = [            # I get to this in a second
          'pandas',
          'numpy',
          'scikit-learn'
      ],
  classifiers=[
    'Development Status :: 5 - Production/Stable',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3.9'      #Specify which pyhton versions that you want to support
  ],
)
